@@ -196,36 +196,53 @@ class DataLoader:
         self._collate_fn = collate_fn
         self._prefetch_factor = prefetch_factor
         self._num_workers = num_workers
+        self.set_format(format)
+        self._padding_value = padding_value
+        self._gen = PrefetchGenerator(self.dataset.data, num_prefetch=self._prefetch_factor*self._batch_size, num_workers=self._num_workers, shuffle=self._shuffle, processing_func=self._collate_fn, name="DataLoader")
+
+    def set_format(self, format):
         self._format = format if format in {"tf", "tensorflow", "torch", "pytorch"} else "numpy"
         if self._format in {"tf", "tensorflow"}:
             if is_available("tensorflow"):
                 import tensorflow as tf
+            else:
+                import tensorflow as tf  # raise
         elif self._format in {"torch", "pytorch"}:
             if is_available("torch"):
                 import torch
-        self._padding_value = padding_value
-        self._gen = PrefetchGenerator(self.dataset.data, num_prefetch=self._prefetch_factor*self._batch_size, num_workers=self._num_workers, shuffle=self._shuffle, processing_func=self._collate_fn, name="DataLoader")
+            else:
+                import torch  # raise
 
     def as_format(self, batch):
         if is_available("numpy"):
             ranks = [len(b) for b in batch]
+            keys = [list(b.keys()) for b in batch]
+            types = [[type(_b) for _b in b.values()] for b in batch]
             rank = np.unique(ranks)
             if len(rank) == 1:
                 rank = rank[0]
+                keys = keys[0]
+                types = types[0]
                 datas = [[] for _ in range(rank)]
                 for b in batch:
                     for i in range(rank):
-                        datas[i].append(b[i])
+                        datas[i].append(b[keys[i]])
                 for i, data in enumerate(datas):
-                    # FIXME: shape 구하는 로직 수정. numpy.ndarray 아니면 동작 안하는데 list, tuple도 처리하도록
-                    # 학습에 사용할 정보 외에 나머지를 dict로 넣어주면 이것도 처리해주면 좋겠음
-                    max_len = np.amax(np.array([d.shape if hasattr(d, "__len__") else 1 for d in data]), axis=0)
-                    min_len = np.amin(np.array([d.shape if hasattr(d, "__len__") else 1 for d in data]), axis=0)
-                    if np.all(max_len == min_len):
-                        datas[i] = np.array(data)
-                    else:
-                        padded_begin = np.zeros(max_len.shape).astype(np.int32)
-                        datas[i] = np.array([np.pad(d, np.stack((padded_begin, max_len - d.shape)).T, constant_values=self._padding_value) for d in data])                        
+                    if types[i] in {np.ndarray, list, tuple, int, float}:
+                        max_len = np.amax(np.array([len(d) if hasattr(d, "__len__") else 1 for d in data]), axis=0)
+                        min_len = np.amin(np.array([len(d) if hasattr(d, "__len__") else 1 for d in data]), axis=0)
+                        if np.all(max_len == min_len):
+                            datas[i] = np.array(data)
+                        else:
+                            padded_begin = np.zeros(max_len.shape).astype(np.int32)
+                            datas[i] = np.array([np.pad(d, np.stack((padded_begin, max_len - len(d))).T, constant_values=self._padding_value) for d in data])
+                    elif types[i] is dict:
+                        tmp_dict = {k: [] for k in data[0].keys()}
+                        for d in data:
+                            for k, v in d.items():
+                                tmp_dict[k].append(v)
+                        datas[i] = tmp_dict
+                datas = {k: v for k, v in zip(keys, datas)}
             else:
                 raise ValueError("collate_fn에서 출력하는 데이터의 개수가 동일하지 않습니다.")
         else:
@@ -233,15 +250,17 @@ class DataLoader:
         if self._format == "tf":
             if is_available("tensorflow"):
                 import tensorflow as tf
-                for i in range(len(datas)):
-                    datas[i] = tf.convert_to_tensor(datas[i])
+                for k, v in datas.items():
+                    if isinstance(v, np.ndarray):
+                        datas[k] = tf.convert_to_tensor(v)
             else:
                 raise ModuleNotFoundError("tensorflow")
         elif self._format == "torch":
             if is_available("torch"):
                 import torch
-                for i in range(len(datas)):
-                    datas[i] = torch.tensor(datas[i])
+                for k, v in datas.items():
+                    if isinstance(v, np.ndarray):
+                        datas[k] = torch.tensor(v)
             else:
                 raise ModuleNotFoundError("torch")
         return datas
